@@ -409,10 +409,14 @@ matern.operators <- function(kappa = NULL,
     stop("You should give either the dimension d, the mesh or graph!")
   }
 
+  mesh_1d <- NULL
+
   if ((is.null(C) || is.null(G)) && is.null(mesh) && is.null(graph) && (is.null(loc_mesh) || d != 1)) {
     stop("You should either provide mesh, graph, or provide both C *and* G!")
   }
 
+  mesh_1d <- NULL
+
   if (!is.null(loc_mesh) && d != 1) {
     stop("loc_mesh only works with dimension 1.")
   } else if (!is.null(loc_mesh)) {
@@ -587,26 +591,6 @@ matern.operators <- function(kappa = NULL,
   #   tau <- 1 / sigma
   # }
 
-  if (!is.null(mesh)) {
-    make_A <- function(loc) {
-      # return(INLA::inla.spde.make.A(mesh = mesh, loc = loc))
-      # return(fmesher::fm_basis(x = mesh, loc = loc))
-      return(fm_basis(x = mesh, loc = loc))
-    }
-  } else if (!is.null(graph)) {
-    make_A <- function(loc) {
-      return(graph$fem_basis(loc))
-    }
-  } else if (!is.null(loc_mesh) && d == 1) {
-    make_A <- function(loc) {
-      # return(rSPDE::rSPDE.A1d(x = loc_mesh, loc = loc))
-      # return(fmesher::fm_basis(x = mesh_1d, loc = loc))
-      return(fm_basis(x = mesh_1d, loc = loc))
-    }
-  } else {
-    make_A <- NULL
-  }
-
   if (type == "operator") {
     beta <- (nu + d / 2) / 2
     operators <- fractional.operators(
@@ -631,26 +615,13 @@ matern.operators <- function(kappa = NULL,
     output$parameterization <- parameterization
     output$has_mesh <- has_mesh
     output$has_graph <- has_graph
-    output$make_A <- make_A
     output$alpha <- 2 * beta
     output$mesh <- mesh
     output$range_mesh <- range_mesh
     output$graph <- graph
     output$loc_mesh <- loc_mesh
-    output$cov_function_mesh <- function(p, direct = FALSE) {
-      if (is.null(output$make_A)) {
-        stop("Please, create the object using the mesh.")
-      }
-      v <- output$make_A(loc = p)
-      if (direct) {
-        return(output$Pr %*% solve(output$Q, output$Pr %*% t(v)))
-      } else {
-        return(Sigma.mult(output, t(v)))
-      }
-    }
-    output$covariance_mesh <- function() {
-      return(output$Pr %*% solve(output$Q, output$Pr))
-    }
+    output$mesh_1d <- mesh_1d
+    class(output) <- c("matern_operator", class(output))
     return(output)
   } else {
     C <- Matrix::Diagonal(dim(C)[1], rowSums(C))
@@ -669,35 +640,12 @@ matern.operators <- function(kappa = NULL,
     out$parameterization <- parameterization
     out$has_mesh <- has_mesh
     out$has_graph <- has_graph
-    out$make_A <- make_A
     out$mesh <- mesh
     out$range_mesh <- range_mesh
     out$graph <- graph
     out$loc_mesh <- loc_mesh
-    out$cov_function_mesh <- function(p) {
-      if (is.null(out$make_A)) {
-        stop("Please, create the object using the mesh.")
-      }
-      v <- t(out$make_A(loc = p))
-      A <- Matrix::Diagonal(dim(C)[1])
-      if (out$alpha %% 1 == 0) {
-        return((A) %*% solve(out$Q, v))
-      } else {
-        v_bar <- kronecker(matrix(1, nrow = m + 1), v)
-        A_bar <- kronecker(matrix(1, ncol = m + 1), A)
-        return((A_bar) %*% solve(out$Q, v_bar))
-      }
-    }
-
-    out$covariance_mesh <- function() {
-      A <- Matrix::Diagonal(dim(C)[1])
-      if (out$alpha %% 1 == 0) {
-        return((A) %*% solve(out$Q, t(A)))
-      } else {
-        A_bar <- kronecker(matrix(1, ncol = m + 1), A)
-        return((A_bar) %*% solve(out$Q, t(A_bar)))
-      }
-    }
+    out$mesh_1d <- mesh_1d
+    class(out) <- c("matern_operator", class(out))
     return(out)
   }
 }
@@ -1038,7 +986,7 @@ CBrSPDE.matern.operators <- function(C,
   output <- list(
     C = C, G = G, L = L, Ci = Ci, GCi = GCi, Gk = Gk,
     fem_mesh_matrices = fem_mesh_matrices,
-    alpha = alpha, nu = nu, kappa = kappa, range = sqrt(8 * nu) / kappa,
+    alpha = alpha, nu = nu, kappa = kappa, range = sqrt(8 * max(nu,0)) / kappa,
     tau = tau, m = m, d = d,
     # sigma = sigma,
     logdetL = logdetL, logdetC = logdetC,
@@ -1070,7 +1018,8 @@ CBrSPDE.matern.operators <- function(C,
 #' parameter evaluated at the locations
 #' of the mesh used for the finite element discretization of the SPDE.
 #' @param theta Theta parameter that connects B.tau and B.kappa to tau and kappa through a log-linear regression, in case the parameterization is `spde`,
-#' and that connects B.sigma and B.range to tau and kappa in case the parameterization is `matern`.
+#' and that connects B.sigma and B.range to tau and kappa in case the parameterization is `matern`. When both tau and kappa are constant after this
+#' transformation, `spde.matern.operators()` delegates to `matern.operators()`.
 #' @param B.sigma Matrix with specification of log-linear model for \eqn{\sigma}. Will be used if `parameterization = 'matern'`.
 #' @param B.range Matrix with specification of log-linear model for \eqn{\rho}, which is a range-like parameter (it is exactly the range parameter in the stationary case). Will be used if `parameterization = 'matern'`.
 #' @param parameterization Which parameterization to use? `matern` uses range, std. deviation and nu (smoothness). `spde` uses kappa, tau and nu (smoothness). The default is `matern`.
@@ -1172,6 +1121,11 @@ spde.matern.operators <- function(kappa = NULL,
                                     "chebfun",
                                     "chebfunLB"
                                   )) {
+  is_constant_param <- function(x) {
+    x <- as.numeric(x)
+    length(unique(x)) == 1
+  }
+
   type <- type[[1]]
   if (!type %in% c("covariance", "operator")) {
     stop("The type should be 'covariance' or 'operator'!")
@@ -1216,6 +1170,8 @@ spde.matern.operators <- function(kappa = NULL,
     stop("You should either provide mesh, graph, or provide both C *and* G!")
   }
 
+  mesh_1d <- NULL
+
   if (!is.null(loc_mesh) && d != 1) {
     stop("loc_mesh only works with dimension 1.")
   } else if (!is.null(loc_mesh)) {
@@ -1322,20 +1278,17 @@ spde.matern.operators <- function(kappa = NULL,
     #   B.tau <- -B.sigma
     # }
 
-    new_theta <- c(1, theta)
+    if (is.null(tau) || is.null(kappa)) {
+      new_theta <- c(1, theta)
 
-    tau <- c(exp(B.tau %*% new_theta))
-    kappa <- c(exp(B.kappa %*% new_theta))
+      tau <- c(exp(B.tau %*% new_theta))
+      kappa <- c(exp(B.kappa %*% new_theta))
+    }
   } else if (is.null(kappa) || is.null(tau)) {
-    if (any(dim(B.tau) != dim(B.kappa))) {
-      stop("B.tau and B.kappa must have the same dimensions!")
-    }
-    if (any(dim(B.sigma) != dim(B.range))) {
-      stop("B.sigma and B.range must have the same dimensions!")
-    }
-
-
     if (parameterization == "matern") {
+      if (any(dim(B.sigma) != dim(B.range))) {
+        stop("B.sigma and B.range must have the same dimensions!")
+      }
       if (is.null(nu)) {
         if(d == 1){
           nu <- 0.75
@@ -1361,6 +1314,9 @@ spde.matern.operators <- function(kappa = NULL,
 
       alpha <- nu + d / 2
     } else {
+      if (any(dim(B.tau) != dim(B.kappa))) {
+        stop("B.tau and B.kappa must have the same dimensions!")
+      }
       B.tau <- prepare_B_matrices(
         B.tau, ncol(C),
         ncol(B.tau) - 1
@@ -1431,22 +1387,45 @@ spde.matern.operators <- function(kappa = NULL,
     }
   }
 
-  if (!is.null(mesh)) {
-    make_A <- function(loc) {
-      # return(INLA::inla.spde.make.A(mesh = mesh, loc = loc))
-      # return(fmesher::fm_basis(x = mesh, loc=loc))
-      return(fm_basis(x = mesh, loc = loc))
+  if (!is.null(tau) && !is.null(kappa) &&
+      is_constant_param(tau) && is_constant_param(kappa)) {
+    if (parameterization == "spde") {
+      return(matern.operators(
+        kappa = as.numeric(kappa)[1],
+        tau = as.numeric(tau)[1],
+        alpha = alpha,
+        G = G,
+        C = C,
+        d = d,
+        mesh = mesh,
+        graph = graph,
+        range_mesh = range_mesh,
+        loc_mesh = loc_mesh,
+        m = m,
+        type = type,
+        parameterization = "spde",
+        type_rational_approximation = type_rational_approximation
+      ))
     }
-  } else if (!is.null(graph)) {
-    make_A <- function(loc) {
-      return(graph$fem_basis(loc))
-    }
-  } else if (!is.null(loc_mesh) && d == 1) {
-    make_A <- function(loc) {
-      return(rspde.make.A(mesh = loc_mesh, loc = loc))
-    }
-  } else {
-    make_A <- NULL
+    range <- sqrt(8 * nu) / as.numeric(kappa)[1]
+    sigma <- sqrt(gamma(nu) / (as.numeric(tau)[1]^2 * as.numeric(kappa)[1]^(2 * nu) *
+      (4 * pi)^(d / 2) * gamma(nu + d / 2)))
+    return(matern.operators(
+      range = range,
+      sigma = sigma,
+      nu = nu,
+      G = G,
+      C = C,
+      d = d,
+      mesh = mesh,
+      graph = graph,
+      range_mesh = range_mesh,
+      loc_mesh = loc_mesh,
+      m = m,
+      type = type,
+      parameterization = "matern",
+      type_rational_approximation = type_rational_approximation
+    ))
   }
 
   if (nu < 0) {
@@ -1478,28 +1457,15 @@ spde.matern.operators <- function(kappa = NULL,
     output$type <- "Matern SPDE approximation"
     output$has_mesh <- has_mesh
     output$has_graph <- has_graph
-    output$make_A <- make_A
     output$mesh <- mesh
     output$range_mesh <- range_mesh
     output$graph <- graph
     output$loc_mesh <- loc_mesh
+    output$mesh_1d <- mesh_1d
     output$B.sigma <- B.sigma
     output$B.range <- B.range
     output$parameterization <- parameterization
-    output$cov_function_mesh <- function(p, direct = FALSE) {
-      if (is.null(output$make_A)) {
-        stop("Please, create the object using the mesh.")
-      }
-      v <- output$make_A(loc = p)
-      if (direct) {
-        return(output$Pr %*% solve(output$Q, output$Pr %*% t(v)))
-      } else {
-        return(Sigma.mult(output, t(v)))
-      }
-    }
-    output$covariance_mesh <- function() {
-      return(output$Pr %*% solve(output$Q, output$Pr))
-    }
+    class(output) <- c("spde_matern_operator", class(output))
   } else {
     type_rational_approximation <- type_rational_approximation[[1]]
     m_alpha <- floor(alpha)
@@ -1597,7 +1563,6 @@ spde.matern.operators <- function(kappa = NULL,
     output$type <- "Covariance-Based Matern SPDE Approximation"
     output$has_mesh <- has_mesh
     output$has_graph <- has_graph
-    output$make_A <- make_A
     output$mesh <- mesh
     output$range_mesh <- range_mesh
     output$graph <- graph
@@ -1605,31 +1570,8 @@ spde.matern.operators <- function(kappa = NULL,
     output$B.sigma <- B.sigma
     output$B.range <- B.range
     output$parameterization <- parameterization
-    output$cov_function_mesh <- function(p) {
-      if (is.null(output$make_A)) {
-        stop("Please, create the object using the mesh.")
-      }
-      v <- t(output$make_A(loc = p))
-      A <- Matrix::Diagonal(dim(C)[1])
-      if (output$alpha %% 1 == 0) {
-        return((A) %*% solve(output$Q, v))
-      } else {
-        v_bar <- kronecker(matrix(1, nrow = m + 1), v)
-        A_bar <- kronecker(matrix(1, ncol = m + 1), A)
-        return((A_bar) %*% solve(output$Q, v_bar))
-      }
-    }
-    class(output) <- "CBrSPDEobj"
-  }
-
-  output$covariance_mesh <- function() {
-    A <- Matrix::Diagonal(dim(C)[1])
-    if (output$alpha %% 1 == 0) {
-      return((A) %*% solve(output$Q, t(A)))
-    } else {
-      A_bar <- kronecker(matrix(1, ncol = m + 1), A)
-      return((A_bar) %*% solve(output$Q, t(A_bar)))
-    }
+    output$mesh_1d <- mesh_1d
+    class(output) <- c("spde_matern_operator", "CBrSPDEobj")
   }
 
   return(output)
@@ -1803,35 +1745,7 @@ matern2d.operators <- function(hx = NULL,
         out$Hxy <- fem$Hxy
     }
 
-    out$make_A <- function(loc) {
-        A <- fm_basis(x = mesh, loc = loc)
-        if(out$alpha %% 1 == 0) {
-            return(A)
-        } else {
-            return(kronecker(matrix(1, ncol = m + 1), A))
-        }
-    }
-
-    out$cov_function_mesh <- function(p) {
-        v <- t(out$make_A(loc = p))
-        A <- Matrix::Diagonal(dim(fem$C)[1])
-        if(out$alpha %% 1 != 0) {
-            A <- kronecker(matrix(1, ncol = m + 1), A)
-        }
-        return((A) %*% solve(out$Q, v))
-    }
-
-    out$covariance_mesh <- function() {
-        A <- Matrix::Diagonal(dim(C)[1])
-        if (out$alpha %% 1 == 0) {
-            return((A) %*% solve(out$Q, t(A)))
-        } else {
-            A_bar <- kronecker(matrix(1, ncol = m + 1), A)
-            return((A_bar) %*% solve(out$Q, t(A_bar)))
-        }
-    }
-
-    class(out) <- "CBrSPDEobj2d"
+    class(out) <- c("matern2d_operator", "CBrSPDEobj2d")
 
     return(out)
 }
@@ -1891,3 +1805,200 @@ CBrSPDE.L.precision <- function(L, tau = 1,
 
     return(Q)
 }
+
+#' Projection matrix for model objects
+#'
+#' Generic for computing the projection matrix that links observation locations
+#' to model mesh nodes.
+#'
+#' @param object A model object.
+#' @param ... Additional arguments passed to methods.
+#' @export
+make_A <- function(object, ...) {
+  UseMethod("make_A")
+}
+
+#' @export
+#' @method make_A default
+make_A.default <- function(object, ...) {
+  if (is.function(object$make_A)) {
+    return(object$make_A(...))
+  }
+  stop("make_A is not available for this object.")
+}
+
+#' @export
+#' @method make_A matern_operator
+make_A.matern_operator <- function(object, loc, ...) {
+  if (!is.null(object$mesh)) {
+    return(fm_basis(x = object$mesh, loc = loc))
+  }
+  if (!is.null(object$graph)) {
+    return(object$graph$fem_basis(loc))
+  }
+  if (!is.null(object$loc_mesh) && object$d == 1) {
+    mesh_1d <- object$mesh_1d
+    if (is.null(mesh_1d)) {
+      mesh_1d <- fm_mesh_1d(object$loc_mesh)
+    }
+    return(fm_basis(x = mesh_1d, loc = loc))
+  }
+  stop("Please, create the object using the mesh.")
+}
+
+#' @export
+#' @method make_A spde_matern_operator
+make_A.spde_matern_operator <- function(object, loc, ...) {
+  if (!is.null(object$mesh)) {
+    return(fm_basis(x = object$mesh, loc = loc))
+  }
+  if (!is.null(object$graph)) {
+    return(object$graph$fem_basis(loc))
+  }
+  if (!is.null(object$loc_mesh) && object$d == 1) {
+    return(rspde.make.A(mesh = object$loc_mesh, loc = loc))
+  }
+  stop("Please, create the object using the mesh.")
+}
+
+#' @export
+#' @method make_A matern2d_operator
+make_A.matern2d_operator <- function(object, loc, ...) {
+  A <- fm_basis(x = object$mesh, loc = loc)
+  if (object$alpha %% 1 == 0) {
+    return(A)
+  }
+  return(kronecker(matrix(1, ncol = object$m + 1), A))
+}
+
+#' Covariance between mesh nodes and locations
+#'
+#' Generic for computing the covariance between mesh nodes and locations for
+#' model objects.
+#'
+#' @param object A model object.
+#' @param ... Additional arguments passed to methods.
+#' @export
+cov_function_mesh <- function(object, ...) {
+  UseMethod("cov_function_mesh")
+}
+
+#' @export
+#' @method cov_function_mesh default
+cov_function_mesh.default <- function(object, ...) {
+  if (is.function(object$cov_function_mesh)) {
+    return(object$cov_function_mesh(...))
+  }
+  stop("cov_function_mesh is not available for this object.")
+}
+
+#' @export
+#' @method cov_function_mesh matern_operator
+cov_function_mesh.matern_operator <- function(object, p, direct = FALSE, ...) {
+  if (inherits(object, "rSPDEobj")) {
+    v <- make_A(object, loc = p)
+    if (direct) {
+      return(object$Pr %*% solve(object$Q, object$Pr %*% t(v)))
+    }
+    return(Sigma.mult(object, t(v)))
+  }
+  v <- t(make_A(object, loc = p))
+  A <- Matrix::Diagonal(dim(object$C)[1])
+  if (object$alpha %% 1 == 0) {
+    return(A %*% solve(object$Q, v))
+  }
+  v_bar <- kronecker(matrix(1, nrow = object$m + 1), v)
+  A_bar <- kronecker(matrix(1, ncol = object$m + 1), A)
+  return(A_bar %*% solve(object$Q, v_bar))
+}
+
+#' @export
+#' @method cov_function_mesh spde_matern_operator
+cov_function_mesh.spde_matern_operator <- function(object, p, direct = FALSE, ...) {
+  if (inherits(object, "rSPDEobj")) {
+    v <- make_A(object, loc = p)
+    if (direct) {
+      return(object$Pr %*% solve(object$Q, object$Pr %*% t(v)))
+    }
+    return(Sigma.mult(object, t(v)))
+  }
+  v <- t(make_A(object, loc = p))
+  A <- Matrix::Diagonal(dim(object$C)[1])
+  if (object$alpha %% 1 == 0) {
+    return(A %*% solve(object$Q, v))
+  }
+  v_bar <- kronecker(matrix(1, nrow = object$m + 1), v)
+  A_bar <- kronecker(matrix(1, ncol = object$m + 1), A)
+  return(A_bar %*% solve(object$Q, v_bar))
+}
+
+#' @export
+#' @method cov_function_mesh matern2d_operator
+cov_function_mesh.matern2d_operator <- function(object, p, ...) {
+  v <- t(make_A(object, loc = p))
+  A <- Matrix::Diagonal(dim(object$fem$C)[1])
+  if (object$alpha %% 1 != 0) {
+    A <- kronecker(matrix(1, ncol = object$m + 1), A)
+  }
+  return(A %*% solve(object$Q, v))
+}
+
+#' Covariance between mesh nodes
+#'
+#' Generic for computing the covariance between mesh nodes for model objects.
+#'
+#' @param object A model object.
+#' @param ... Additional arguments passed to methods.
+#' @export
+covariance_mesh <- function(object, ...) {
+  UseMethod("covariance_mesh")
+}
+
+#' @export
+#' @method covariance_mesh default
+covariance_mesh.default <- function(object, ...) {
+  if (is.function(object$covariance_mesh)) {
+    return(object$covariance_mesh(...))
+  }
+  stop("covariance_mesh is not available for this object.")
+}
+
+#' @export
+#' @method covariance_mesh matern_operator
+covariance_mesh.matern_operator <- function(object, ...) {
+  if (inherits(object, "rSPDEobj")) {
+    return(object$Pr %*% solve(object$Q, object$Pr))
+  }
+  A <- Matrix::Diagonal(dim(object$C)[1])
+  if (object$alpha %% 1 == 0) {
+    return(A %*% solve(object$Q, t(A)))
+  }
+  A_bar <- kronecker(matrix(1, ncol = object$m + 1), A)
+  return(A_bar %*% solve(object$Q, t(A_bar)))
+}
+
+#' @export
+#' @method covariance_mesh spde_matern_operator
+covariance_mesh.spde_matern_operator <- function(object, ...) {
+  if (inherits(object, "rSPDEobj")) {
+    return(object$Pr %*% solve(object$Q, object$Pr))
+  }
+  A <- Matrix::Diagonal(dim(object$C)[1])
+  if (object$alpha %% 1 == 0) {
+    return(A %*% solve(object$Q, t(A)))
+  }
+  A_bar <- kronecker(matrix(1, ncol = object$m + 1), A)
+  return(A_bar %*% solve(object$Q, t(A_bar)))
+}
+
+#' @export
+#' @method covariance_mesh matern2d_operator
+covariance_mesh.matern2d_operator <- function(object, ...) {
+  C <- Matrix::Diagonal(dim(object$fem$C)[1], rowSums(object$fem$C))
+  A <- Matrix::Diagonal(dim(C)[1])
+  if (object$alpha %% 1 == 0) {
+    return(A %*% solve(object$Q, t(A)))
+  }
+  A_bar <- kronecker(matrix(1, ncol = object$m + 1), A)
+  return(A_bar %*% solve(object$Q, t(A_bar)))
+}
